"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DEFAULT_EMBED_HEIGHT = exports.DEFAULT_EMBED_WIDTH = exports.URL_MAX_LENGTH = exports.getV2BasePath = exports.getThoughtSpotHost = void 0;
const errors_1 = require("./errors");
/**
 * Copyright (c) 2023
 *
 * Utilities related to reading configuration objects
 * @summary Config-related utils
 * @author Ayon Ghosh <ayon.ghosh@thoughtspot.com>
 */
const urlRegex = new RegExp([
    '(^(https?:)//)?',
    '(([^:/?#]*)(?::([0-9]+))?)',
    '(/{0,1}[^?#]*)',
    '(\\?[^#]*|)',
    '(#.*|)$', // hash
].join(''));
/**
 * Parse and construct the ThoughtSpot hostname or IP address
 * from the embed configuration object.
 * @param config
 */
const getThoughtSpotHost = (config) => {
    if (!config.thoughtSpotHost) {
        throw new Error(errors_1.ERROR_MESSAGE.INVALID_THOUGHTSPOT_HOST);
    }
    const urlParts = config.thoughtSpotHost.match(urlRegex);
    if (!urlParts) {
        throw new Error(errors_1.ERROR_MESSAGE.INVALID_THOUGHTSPOT_HOST);
    }
    const protocol = urlParts[2] || window.location.protocol;
    const host = urlParts[3];
    let path = urlParts[6];
    // Lose the trailing / if any
    if (path.charAt(path.length - 1) === '/') {
        path = path.substring(0, path.length - 1);
    }
    // const urlParams = urlParts[7];
    // const hash = urlParts[8];
    return `${protocol}//${host}${path}`;
};
exports.getThoughtSpotHost = getThoughtSpotHost;
const getV2BasePath = (config) => {
    if (config.basepath) {
        return config.basepath;
    }
    const tsHost = (0, exports.getThoughtSpotHost)(config);
    // This is to handle when e2e's. Search is run on pods for
    // comp-blink-test-pipeline with baseUrl=https://localhost:8443.
    // This is to handle when the developer is developing in their local
    // environment.
    if (tsHost.includes('://localhost') && !tsHost.includes(':8443')) {
        return '';
    }
    return 'v2';
};
exports.getV2BasePath = getV2BasePath;
/**
 * It is a good idea to keep URLs under 2000 chars.
 * If this is ever breached, since we pass view configuration through
 * URL params, we would like to log a warning.
 * Reference: https://stackoverflow.com/questions/417142/what-is-the-maximum-length-of-a-url-in-different-browsers
 */
exports.URL_MAX_LENGTH = 2000;
/**
 * The default CSS dimensions of the embedded app
 */
exports.DEFAULT_EMBED_WIDTH = '100%';
exports.DEFAULT_EMBED_HEIGHT = '100%';
//# sourceMappingURL=config.js.map